"""This script has multiple utilities to work with HTSeq count files.

Of particular note, the `header` subcommand can be used to add a header to HTSeq count
files so that they can be used by WARDEN. The logic is simple: it adds a constant column
name for the gene name and then looks at the file name to add the sample column.

Further, the `samplesheet` subcommand can be used to quickly bootstrap up a WARDEN
samplesheet.
"""

import argparse
import os

from glob import glob
from typing import List
from itertools import combinations

def main():
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    subparsers = parser.add_subparsers(dest="subcommand", required=True)

    header = subparsers.add_parser("header", description="Headers an HTSeq count file for use in WARDEN.")
    header.add_argument("input_file", help="HTSeq-count file to add a header to.")
    header.add_argument("output_file", help="File to write the result to.")
    header.add_argument(
        "-g", "--gene-column-name",
        default="Gene",
        help="Constant value to set the gene column to."
    )

    samplesheet = subparsers.add_parser("samplesheet", description="Prepares a samplesheet for WARDEN.")
    samplesheet.add_argument("output_file", help="File to write the result to.")
    samplesheet.add_argument("-d", "--dirs", nargs="+", required=True, help="Directories containing samples within their respective groups.")

    args = parser.parse_args()
    kwargs = vars(args)
    globals()[kwargs.pop('subcommand')](**kwargs)

def samplename_from_file(filename: str):
    return os.path.basename(filename).split(".")[0] if "." in filename else filename

def header(
    input_file: str, output_file: str, gene_column_name: str
):
    sample_name = samplename_from_file(input_file)

    with open(input_file, "r") as f:
        contents = f.read()

    parent_dir = os.path.dirname(output_file)
    if not os.path.exists(parent_dir):
        os.makedirs(parent_dir)

    with open(output_file, "w") as f:
        f.write(f"{gene_column_name}\t{sample_name}\n")
        f.write(contents)


def samplesheet(dirs: List[str], output_file: str):
    if not len(dirs) >= 2:
        raise RuntimeError("Must have at least two groups to create a samplesheet for!")

    for dir in dirs:
        if not os.path.isdir(dir):
            raise RuntimeError(f"{dir} is not a directory.")

        if "-" in dir:
            raise RuntimeError(f"Folders cannot have a dash in the name: {dir}")

        if "," in dir:
            raise RuntimeError(f"Folders cannot have a comma in the name: {dir}")


    with open(output_file, "w") as f:
        f.write("Sample\tPhenotype\tReadfile1\tReadfile2\n")
        for dir in dirs:
            for detected_file in glob(dir + "/*"):
                bn = os.path.basename(detected_file)
                sample_name = samplename_from_file(bn)
                f.write(f"{sample_name}\t{dir}\t{bn}\t-\n")

        f.write("#comparisons=")
        f.write(",".join([x + "-" + y for (x, y) in combinations(dirs, 2)]))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import argparse
import json
import sys
from typing import List
from subprocess import check_output
from collections import defaultdict

store = defaultdict(list)

LOG = False
BAD_FILES_FOLDER = "/bad-files/"
PRIORITIES = [
    "Clinical Pilot",
    "Pediatric Cancer Genome Project (PCGP)",
    "Real Time Clinical Genomics (RTCG)",
    "Genomes 4 Kids (G4K)",
    "Childhood Solid Tumor Network (CSTN)",
    "Pan-Acute Lymphoblastic Leukemia (PanALL)",
    "Pediatric Therapy-related Myeloid Neoplasms (tMN)",
    "Pediatric Brain Tumor Program (PBTP)",
    "Pediatric Acute Myeloid Leukemia (PedAML)",
    "Genomics and transcriptomics of relapsed pediatric AML (RPAML)",
]


def simple_log(msg: str = ""):
    if LOG:
        sys.stderr.write(f"LOG: {msg}\n")
        sys.stderr.flush()


def simple_error(msg: str, suggest_contact: bool = True):
    sys.stderr.write(f"\n\nERROR: {msg}")
    if suggest_contact:
        sys.stderr.write(" Please contact us at support@stjude.cloud with this error.")
    sys.stderr.flush()
    sys.exit(1)


def get_best_file(
    fn: str, file_ids: List[str], priority_list: List[str] = PRIORITIES
) -> str:
    """Gets the best file given the dataset priorities given in priorities

    Args:
        file_id (List[str]): list of DNAnexus file ids that have a similar name.
        priorities (List[str]): list of datasets to prioritize

    Raises:
        RuntimeError: file does not have the correct properties annotated.

    Returns:
        str: best file id to use based on priority_list
    """

    file_id_to_datasets = {}
    for fid in file_ids:
        full_describe = check_output(f"dx describe {fid} --json", shell=True)
        datasets = json.loads(full_describe).get("properties", {}).get("sj_datasets")
        if not datasets:
            raise RuntimeError(
                f"File {fid} doesn't have the correct properties annotated! "
                + "Did you get this file from St. Jude Cloud? If so, please "
                + "contact us with this error at support@stjude.cloud.",
                suggest_contact=False,
            )

        file_id_to_datasets[fid] = set([e.strip() for e in datasets.split(";")])

    # first, ensure every file belongs to *at least* one dataset above.
    # if any file belongs to a new dataset, then out of an abundance of
    # caution, we will error and ask the user to explicitly add that
    # dataset to the priority_list list.

    for file_id, datasets in file_id_to_datasets.items():
        if not any([dataset in priority_list for dataset in datasets]):
            simple_error(
                f"File {fid} doesn't contain a dataset tag in the known priority "
                + "list. This likely means that a new dataset has been released "
                + "to St. Jude Cloud since this script was written. Out of an "
                + "abundance of caution, we ask that you explicitly "
                + "update the PRIORITIES variable at the top of this script to "
                + f"include one of the following values: {datasets}. "
                + "\n\nWe would also appreciate it if you reached out to us at "
                + "support@stjude.cloud with this error so we can fix it for future "
                + "users.",
                suggest_contact=False,
            )

    simple_log()
    simple_log(f"== {fn} ==")

    hooked_file = None
    for candidate_dataset in priority_list:
        simple_log(f'  [*] Looking for dataset "{candidate_dataset}".')
        for file_id, actual_datasets in file_id_to_datasets.items():
            contains = any([e == candidate_dataset for e in actual_datasets])
            simple_log(f"    - {file_id} => {actual_datasets} (contains: {contains})")
            if contains:
                if not hooked_file:
                    hooked_file = file_id
                else:
                    # a file that was attached to this dataset already existed, and now we've found
                    # a second one. I don't expect this to ever happen in practice (two files in the
                    # same dataset with the same name), so if it does, we should email support@stjude.cloud
                    simple_error(
                        f"More than one file id had the same dataset and same name: {file_id_to_datasets}."
                        + "This is unexpected."
                    )

        if hooked_file:
            simple_log(f"    - Found file in {candidate_dataset}: {hooked_file}.")
            break

    if not hooked_file:
        raise simple_error(
            f"No files with datasets in the priority list were found: {file_id_to_datasets}."
        )

    return hooked_file


def main():
    parser = argparse.ArgumentParser(
        description="Deduplicates samples in DNAnexus based on the project priority " + \
             "the St. Jude Cloud team uses."
    )
    parser.add_argument("dxids", nargs="+", type=str, help="DNAnexus file ids to deduplicate")
    args = parser.parse_args()

    for line in [l.strip() for l in args.dxids]:
        if ":" not in line:
            continue

        fn, fid = [s.strip() for s in line.split(":")]
        store[fn].append(fid)

    print(f"dx mkdir {BAD_FILES_FOLDER}")
    for fn, fids in store.items():
        best_file_id = get_best_file(fn, fids)
        files_to_remove = fids.copy()
        files_to_remove.remove(best_file_id)
        print(f"dx mv {' '.join(list(files_to_remove))} {BAD_FILES_FOLDER}")

if __name__ == "__main__":
    main()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['stjudecloud-utilities']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['deduplicate-feature-counts = '
                     'stjudecloud-utilities.deduplicate_feature_counts:main',
                     'gtf-to-rseqc-bed = '
                     'stjudecloud-utilities.gtf_to_rseqc_bed:main',
                     'warden-counts-utils = '
                     'stjudecloud-utilities.warden_counts_utils:main']}

setup_kwargs = {
    'name': 'stjudecloud-utilities',
    'version': '1.1.0',
    'description': 'A set of utilities used on the St. Jude Cloud project.',
    'long_description': None,
    'author': 'St. Jude Cloud',
    'author_email': 'support@stjude.cloud',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""Convert a Gencode GTF into the custom bed12 format used by the RSeQC software package.

Prints the new BED file to stdout.
"""

import gzip
import re
import argparse
from typing import DefaultDict


class Tx:
    def __init__(self, tx_start=None, tx_end=None, contig=None):
        self.start = tx_start
        self.end = tx_end
        self.contig = contig
        self.strand = None
        self.starts = []
        self.ends = []


def main():
    parser = argparse.ArgumentParser(
        description="Converts a GTF file to an RSeQC bed12 format."
    )
    parser.add_argument("gtf", help="GTF to convert")
    args = parser.parse_args()

    filename = args.gtf
    gzipped = False
    if "gz" in filename.split(".")[-1]:
        handle = gzip.open(filename)
        gzipped = True
    else:
        handle = open(filename, "r")

    attr_regexes = [r"(\S+)=(\S+)", r"(\S+) \"(\S+)\""]
    transcripts = DefaultDict(Tx)
    for line in handle:
        if gzipped:
            line = line.decode("utf-8")
        if line[0] == "#":
            continue
        [
            seqname,
            _source,
            feature,
            start,
            end,
            _score,
            strand,
            _frame,
            attribute,
        ] = line.split("\t")

        if feature != "transcript" and feature != "exon":
            continue
        attribute = attribute.replace(';"', '"').replace(
            ";-", "-"
        )  # correct error in ensemble 78 release
        attributes = {}
        for attr_raw in [s.strip() for s in attribute.split(";")]:
            if not attr_raw or attr_raw == "":
                continue

            for regex in attr_regexes:
                match = re.match(regex, attr_raw)
                if match:
                    key, value = match.group(1), match.group(2)
                    attributes[key.strip()] = value.strip()

        tx = transcripts[  # pylint: disable=unsubscriptable-object
            attributes["transcript_id"]
        ]
        if feature == "transcript":
            assert tx.start is None
            tx.start = int(start) - 1
            tx.end = int(end)
            tx.contig = seqname
            if tx.strand:
                assert tx.strand == strand
            tx.strand = strand
        else:
            tx.starts.append(int(start) - 1)
            tx.ends.append(int(end))
            if tx.strand:
                assert tx.strand == strand

    for tx_name, tx in transcripts.items():  # pylint: disable=no-member
        print(
            "\t".join(
                [
                    tx.contig,
                    str(tx.start),
                    str(tx.end),
                    tx_name,
                    "0",
                    tx.strand,
                    ".",
                    ".",
                    ".",
                    str(len(tx.starts)),
                ]
            ),
            end="\t",
        )
        sizes = [str(end - start) for start, end in zip(tx.starts, tx.ends)]
        starts = [str(start - tx.start) for start in tx.starts]
        print(",".join(sizes), end="\t")
        print(",".join(starts))


if __name__ == "__main__":
    main()
